/*
Copyright (C) 2009 Marco Michelino <michelinux@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* This module contains quick and dirty text drawing and menu functions */

#include <wiiuse/wpad.h>

#ifndef TEXT_H
#define TEXT_H

#define MAX_ITEMS 5

#define FONT_SIZE 16

#define MAX_CHAR ((SCREEN_WIDTH - 2 * HORIZONTAL_BORDER) / FONT_SIZE)

#define NUM_LANGUAGES 5

struct menu_item {
  int x;
  int y;
  int x1;
  int y1;
  int highlighted;
};

typedef char *multilang_text[NUM_LANGUAGES];

/* represents a menu with optional prompt */
struct menu {
  /* number of items including prompt */
  int n;
  /* which item is selected (0 = first item after prompt) */
  int selected;
  /* initialized by init_horizontal_menu(), don't touch */
  struct menu_item items[MAX_ITEMS];
  /* 0 if the first item is selectable, 1 if is just a prompt */
  int prompt;
  /* string[item number][language] */
  multilang_text *text;
};

/* string[item number][language] */
extern multilang_text levels_text[];
extern multilang_text player1_movement_text[];
extern multilang_text players_text[];
extern multilang_text clouds_text[];
extern multilang_text sounds_text[];
extern multilang_text start_text[];
extern multilang_text back_text[];
extern multilang_text quit_play_again_text[];
extern multilang_text quit_continue_text[];

/* string[language] */
extern multilang_text title_text;
extern multilang_text connect_nunckuk_text;
extern multilang_text wiimote_not_detected_text;
extern multilang_text switch_on_bb_text;
extern multilang_text pausing_text;
extern multilang_text game_over_text;
extern multilang_text saucers_destroyed_text;

extern int language;

/* inizialization for this module */
void init_text();

/* print a string at the center of givern row */
void draw_centered_string(int row, const char *text, u32 color);

/* print a string on the right of givern row */
void draw_right_aligned_string(int row, const char *text, u32 color);

/* print a translated text at the center of givern row */
void draw_centered_text(int row, const multilang_text text, u32 color);

/* inizialize a menu struct */
void init_horizontal_menu(int row, struct menu *m);

/* highlight and select menu items */
void menu_interaction(int n, struct menu m[], WPADData *wd[], const u32 pressed[]);

void draw_menu(int n, const struct menu m[]);

#endif

