/*
Copyright (C) 2009 Marco Michelino <michelinux@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <grrlib.h>
#include <asndlib.h>
#include <wiiuse/wpad.h>
#include <wiiuse/wiiuse.h>
#include "main.h"
#include "saucer.h"
#include "missile.h"
#include "player.h"
#include "explosion.h"
#include "cursor.h"
#include "text.h"
#include "cloud.h"

#define BACKGROUND 0x000040FF

int level= 1;
int use_sounds= 1;

static u32 pressed[MAX_PLAYERS];
static WPADData *wd[MAX_PLAYERS];
static int quitting= -1;

/* return to loader, power off or reset wii */
static void quit() {
  GRRLIB_Exit();
  ASND_End();

  if (quitting == -1) {
    exit(0);
  } else {
    SYS_ResetSystem(quitting, 0, 0);
  }
}

/* read wiimote data for players with status from min_status to max_status */
static void scan_wiimotes(int min_status, int max_status) {
  int i;
  WPAD_ScanPads();
  for (i= 0; i < MAX_PLAYERS; i++) {
    if (player_status[i] >= min_status && player_status[i] <= max_status) {
      wd[i]= WPAD_Data(i);
      pressed[i]= WPAD_ButtonsDown(i);

      if (pressed[i] & WPAD_BUTTON_HOME || quitting != -1) {
        quit();
      }
      if (pressed[i] & WPAD_BUTTON_MINUS) {
        GRRLIB_ScrShot("Screenshot.png");
      }
    } else {
      wd[i]= NULL;
      pressed[i]= 0;
    }
  }
}

/* return 1 if nunchuk is connected to given wiimote */
static int nunchuk_connected(int wiimote) {
  expansion_t exp;
  exp.type = WPAD_EXP_NUNCHUK;
  WPAD_Expansion(wiimote, &exp);
  if (exp.type == WPAD_EXP_NUNCHUK) {
    return 1;
  } else {
    return 0;
  }
}

/* callback for wii reset button */
void reset_pressed() {
  quitting= SYS_RETURNTOMENU;
}

/* callback for wii and wiimote power button */
void power_pressed() {
  quitting= SYS_POWEROFF;
}

/* play one match of the game */
static void play() {
  int i;
  int killed_saucers[MAX_PLAYERS];
  char saucers_destroyed[MAX_CHAR + 1];
  struct menu menus;
  int pausing= 0;

  init_players();
  init_missiles();
  init_saucers();
  init_clouds();

  for (i= 0; i < MAX_PLAYERS; i++) {
    killed_saucers[i]= 0;
  }

  menus.n= 2;
  menus.text= quit_continue_text;
  menus.prompt= 0;
  init_horizontal_menu(16, &menus);

  /* main game loop */ 
  while (alive_players()) {
    scan_wiimotes(ALIVE_PLAYER, ALIVE_PLAYER);

    for (i= 0; i < MAX_PLAYERS; i++) {
      if (pressed[i] & WPAD_BUTTON_PLUS) {
        pausing= !pausing;
        menus.selected= -1;
      }
    }

    if (!pausing) {
      for (i= 0; i < MAX_PLAYERS; i++) {
        if (player_status[i] == ALIVE_PLAYER && (pressed[i] & WPAD_BUTTON_B)) {
          killed_saucers[i]+= hit_saucers(wd[i]);
        }
      }

      move_players();
      move_saucers();
      move_missiles();
      move_clouds();

      new_saucer();
      new_cloud(0);
    } else {
      menu_interaction(1, &menus, wd, pressed);
      if (menus.selected == 0) {
        quit();
      } else if (menus.selected == 1) {
        pausing= 0;
      }
    }

    GRRLIB_FillScreen(BACKGROUND);
    draw_background_clouds();
    draw_saucers();
    draw_players();
    draw_missiles();
    draw_foreground_clouds();
    if (pausing) {
      draw_centered_text(8, pausing_text, WHITE);
      draw_menu(1, &menus);
    }
    for (i= 0; i < MAX_PLAYERS; i++) {
      if (player_status[i] >= ALIVE_PLAYER) {
        snprintf(saucers_destroyed, MAX_CHAR, "%d", killed_saucers[i]);
        draw_right_aligned_string(i, saucers_destroyed, player_color[i]);
      }
    }
    draw_cursors(wd);
    GRRLIB_Render();
  }

  /* game over */
  i= 0;
  menus.text= quit_play_again_text;
  init_horizontal_menu(16, &menus);
  menus.selected= -1;
  while (menus.selected == -1) {
    scan_wiimotes(ALIVE_PLAYER + 1, DEAD_PLAYER);
    
    for (i= 0; i < MAX_PLAYERS; i++) {
      if (player_status[i] > ALIVE_PLAYER) {
        if (pressed[i] & WPAD_BUTTON_HOME || quitting != -1) {
          quit();
        }
      }
    }

    menu_interaction(1, &menus, wd, pressed);
    if (menus.selected == 0) {
      quit();
    }

    move_clouds();
    move_saucers();
    move_missiles();

    new_cloud(0);

    GRRLIB_FillScreen(BACKGROUND);
    draw_background_clouds();
    draw_saucers();
    draw_players();
    draw_missiles();
    draw_foreground_clouds();
    draw_centered_text(1, game_over_text, WHITE);
    for (i= 0; i < MAX_PLAYERS; i++) {
      if (player_status[i] > ALIVE_PLAYER) {
        snprintf(saucers_destroyed, MAX_CHAR, saucers_destroyed_text[language], i + 1, killed_saucers[i]);
        draw_centered_string(4 + i, saucers_destroyed, WHITE);
      }
    }
    draw_menu(1, &menus);
    draw_cursors(wd);
    GRRLIB_Render();
  }

  return;
}

int main(int argc, char *argv[]) {
  struct menu menus[6];
  int i;
  char s[MAX_CHAR + 1];
  u32 devtype;
  struct menu menu_back[1];

  SYS_SetResetCallback(reset_pressed);
  SYS_SetPowerCallback(power_pressed);
  WPAD_SetPowerButtonCallback(power_pressed);

  GRRLIB_Init();
  ASND_Init(NULL);
  ASND_Pause(0);
  player_status[0]= MISSING_NUNCHUK;
  for (i= 1; i < MAX_PLAYERS; i++) {
    player_status[i]= INACTIVE_PLAYER;
  }
  init_cursor();
  init_explosion();
  init_text();
  init_saucers();

  /* main menu initialization */
  menus[0].n= 4;
  menus[0].selected= level;
  menus[0].text= levels_text;
  menus[0].prompt= 1;
  init_horizontal_menu(4, menus);
  menus[1].n= 3;
  menus[1].selected= 0;
  menus[1].text= player1_movement_text;
  menus[1].prompt= 1;
  init_horizontal_menu(6, menus + 1);
  menus[2].n= 5;
  menus[2].selected= 0;
  menus[2].text= players_text;
  menus[2].prompt= 1;
  init_horizontal_menu(8, menus + 2);
  menus[3].n= 3;
  menus[3].selected= 1;
  menus[3].text= clouds_text;
  menus[3].prompt= 1;
  init_horizontal_menu(10, menus + 3);
  menus[4].n= 3;
  menus[4].selected= 1;
  menus[4].text= sounds_text;
  menus[4].prompt= 1;
  init_horizontal_menu(12, menus + 4);
  menus[5].n= 1;
  menus[5].text= start_text;
  menus[5].prompt= 0;
  init_horizontal_menu(16, menus + 5);

  int all_devices_connected= 0;
  do {

    /* show main menu */
    menus[5].selected= -1;
    while (menus[5].selected == -1) {
      scan_wiimotes(MISSING_NUNCHUK, ALIVE_PLAYER);

      if (pressed[0] & WPAD_BUTTON_HOME || quitting != -1) {
        quit();
      }

      menu_interaction(6, menus, wd, pressed);

      GRRLIB_FillScreen(BACKGROUND);
      draw_centered_text(1, title_text, WHITE);
      draw_active_saucer(HORIZONTAL_BORDER, VERTICAL_BORDER, 0);
      draw_active_saucer(SCREEN_WIDTH - HORIZONTAL_BORDER - SAUCER_WIDTH, VERTICAL_BORDER, 1);
      draw_active_saucer(HORIZONTAL_BORDER, SCREEN_HEIGHT - VERTICAL_BORDER - SAUCER_HEIGHT, 2);
      draw_active_saucer(SCREEN_WIDTH - HORIZONTAL_BORDER - SAUCER_WIDTH, SCREEN_HEIGHT - VERTICAL_BORDER - SAUCER_HEIGHT, 3);
      draw_menu(6, menus);
      draw_cursors(wd);
      GRRLIB_Render();
    }

    level= menus[0].selected;
    use_balance_board= menus[1].selected;
    player_status[0]= MISSING_NUNCHUK;
    for (i= 1; i < MAX_PLAYERS; i++) {
      if (i < menus[2].selected + 1) {
        player_status[i]= MISSING_WIIMOTE;
      } else {
        player_status[i]= INACTIVE_PLAYER;
      }
    }
    use_clouds= menus[3].selected;
    use_sounds= menus[4].selected;
    
    menu_back[0].n= 1;
    menu_back[0].selected= -1;
    menu_back[0].text= back_text;
    menu_back[0].prompt= 0;
    init_horizontal_menu(16, menu_back);

    /* verify choosen input device are all connected */
    while (!all_devices_connected && (menu_back[0].selected == -1)) {
      all_devices_connected= 1;

      scan_wiimotes(MISSING_NUNCHUK, ALIVE_PLAYER);

      for (i= 0; i < MAX_PLAYERS; i++) {
        if (player_status[i] > INACTIVE_PLAYER) {
          if (pressed[i] & WPAD_BUTTON_HOME || quitting != -1) {
            quit();
          }
        }
      }

      /* is player1 ok? */
      if (player_status[0] == MISSING_NUNCHUK) {
        if (use_balance_board == 0) {
          if (nunchuk_connected(0)) {
            player_status[0]= ALIVE_PLAYER;
          } else {
            all_devices_connected= 0;
          }
        } else {
          for (i= 1; i < WPAD_MAX_WIIMOTES; i++) {
            WPAD_Probe(i, &devtype);
            if (devtype == WPAD_EXP_WIIBOARD) {
              use_balance_board= i;
              player_status[0]= ALIVE_PLAYER;
            } else {
              all_devices_connected= 0;
            }
          }
        }
      }

      /* are other active players ok? */
      for (i= 1; i < MAX_PLAYERS; i++) {
        if (player_status[i] == MISSING_WIIMOTE) {
          if (WPAD_Probe(i, &devtype) == WPAD_ERR_NONE) {
            player_status[i]= MISSING_NUNCHUK;
            WPAD_SetDataFormat(i, WPAD_FMT_BTNS_ACC_IR);
            WPAD_SetVRes(i, SCREEN_WIDTH, SCREEN_HEIGHT);
          } else {
            all_devices_connected= 0;
          }
        }
        if (player_status[i] == MISSING_NUNCHUK) {
          if (nunchuk_connected(i)) {
            player_status[i]= ALIVE_PLAYER;
          } else {
            all_devices_connected= 0;
          }
        }
      }

      menu_interaction(1, menu_back, wd, pressed);

      GRRLIB_FillScreen(BACKGROUND);
      if (player_status[0] == MISSING_NUNCHUK) {
        if (use_balance_board) {
          draw_centered_text(7, switch_on_bb_text, WHITE);
        } else {
          snprintf(s, MAX_CHAR, connect_nunckuk_text[language], 1);
          draw_centered_string(7, s, WHITE);
        }
      }
      for (i= 1; i < MAX_PLAYERS; i++) {
        if (player_status[i] == MISSING_WIIMOTE) {
          snprintf(s, MAX_CHAR, wiimote_not_detected_text[language], i + 1);
          draw_centered_string(7 + i, s, WHITE);
        } else if (player_status[i] == MISSING_NUNCHUK) {
          snprintf(s, MAX_CHAR, connect_nunckuk_text[language], i + 1);
          draw_centered_string(7 + i, s, WHITE);
        }
      }
      draw_menu(1, menu_back);
      draw_cursors(wd);
      GRRLIB_Render();
    }

  } while (!all_devices_connected);

  srand(time(NULL));

  /* play the game */
  while (1) play();

  return 0;
}

