/*
Copyright (C) 2009 Marco Michelino <michelinux@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <grrlib.h>
#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "missile.h"
#include "player.h"

#define MISSILE_SPEED 5

#define MISSILE_COLOR 0xFFFF00FF

#define MAX_MISSILES 25

#define INACTIVE_MISSILE 0
#define ACTIVE_MISSILE   1

struct missile {
  int x;
  int y;
  int vx;
  int vy;
  int state;
};

static struct missile missiles[MAX_MISSILES];

void move_missiles() {
  int i;
  for (i= 0; i < MAX_MISSILES; i++) {
    if (missiles[i].state == ACTIVE_MISSILE) {
      missiles[i].x+= missiles[i].vx;
      missiles[i].y+= missiles[i].vy;
      if (hit_players(missiles[i].x, missiles[i].y) ||
          missiles[i].y >= SCREEN_HEIGHT ||
          missiles[i].x < 0 ||
          missiles[i].x > SCREEN_WIDTH) {
        missiles[i].state= INACTIVE_MISSILE;
      }
    }
  }
}

/* inizialization for this module */
void init_missiles() {
  int i;
  for (i= 0; i < MAX_MISSILES; i++) {
    missiles[i].state= INACTIVE_MISSILE;
  }
}

void draw_missiles() {
  int i;
  for (i= 0; i < MAX_MISSILES; i++) {
    if (missiles[i].state == ACTIVE_MISSILE) {
      GRRLIB_Circle(missiles[i].x, missiles[i].y, 2, MISSILE_COLOR, TRUE);
    }
  }
}

/* launch new missile from (x,y) to a random player */
int new_missile(const int x, const int y) {
  int i, j;
  for (i= 0; i < MAX_MISSILES; i++) {
    if (missiles[i].state == INACTIVE_MISSILE) {
      int alive= alive_players();
      if (alive) {
        /* choose random target between alive players */
        int target_player= ((float) alive * rand()) / (RAND_MAX + 1.0);
        int count= 0;
        for (j= 0; j < MAX_PLAYERS; j++) {
          if (player_status[j] == ALIVE_PLAYER) {
            if (count == target_player) {
              int dx= player_x[j] + player_width / 2 - x;
              int dy= PLAYER_Y - y;
              int distance= sqrt(dy * dy + dx * dx);
              missiles[i].x= x;
              missiles[i].y= y;
              missiles[i].vx= (dx * MISSILE_SPEED) / distance;
              missiles[i].vy= (dy * MISSILE_SPEED) / distance;
              missiles[i].state= ACTIVE_MISSILE;
              return 1;;
            }
            count++;
          }
        }
      } else {
        /* no alive players to shoot at */
        return 0;
      }
    }
  }
  /* all missiles fired! */
  return 0;
}

