/*
Copyright (C) 2009 Marco Michelino <michelinux@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <grrlib.h>
#include <wiiuse/wpad.h>
#include "main.h"
#include "player.h"
#include "explosion.h"
#include "player_png.h"

#define PLAYER_WIDTH 88

#define BB_MOVEMENT_AMPLIFICATION 1.3

int player_status[MAX_PLAYERS];
int player_x[MAX_PLAYERS];
int use_balance_board;
unsigned int player_color[MAX_PLAYERS]= {0x5060FFFF, 0xFF3050FF, 0x40FF40FF, 0xE040FFFF};

/* player width depends on game level */
int player_width;
static float player_w_scale[]= {0.75, 0.9, 1};

static GRRLIB_texImg *player_image= NULL;

/* return number of alive players */
int alive_players() {
  int i;
  int alive= 0;
  for (i= 0; i < MAX_PLAYERS; i++) {
    if (player_status[i] == ALIVE_PLAYER) {
      alive++;
    }
  }
  return alive;
}

/* inizialization for this module */
void init_players() {
  int i;
  int n_players;
  int count= 0;

  n_players= active_players();

  player_width= PLAYER_WIDTH * player_w_scale[level];

  for (i= 0; i < MAX_PLAYERS; i++) {
    player_x[i]= ((count + 1) * (SCREEN_WIDTH - n_players * player_width)) / (n_players + 1) + count * player_width;

    if (player_status[i] > MISSING_NUNCHUK) {
      player_status[i]= ALIVE_PLAYER;
      WPAD_Rumble(i, 0);
      count++;
    }
  }

  if (!player_image) {
    player_image= GRRLIB_LoadTexture(player_png);
  }
}

/* let the players go around */
void move_players() {
  int i;
  int movement;
  struct expansion_t data;

  for (i= 0; i < MAX_PLAYERS; i++) {
    if (i == 0 && use_balance_board && player_status[0] == ALIVE_PLAYER) {
      WPAD_Expansion(use_balance_board, &data);
      if (data.type == WPAD_EXP_WIIBOARD) {
        float weight= data.wb.tl + data.wb.tr + data.wb.bl + data.wb.br;
        if (weight < 10) {
          player_x[0]= (SCREEN_WIDTH - player_width) / 2;
        } else {
          player_x[0]= (SCREEN_WIDTH - player_width) * (BB_MOVEMENT_AMPLIFICATION * ((data.wb.tr + data.wb.br) / weight - 0.5) + 0.5);
        }
      }
    } else {
      if (player_status[i] == ALIVE_PLAYER) {
        WPAD_Expansion(i, &data);
        if (data.type == WPAD_EXP_NUNCHUK) {
          movement= (20.0 * (data.nunchuk.js.pos.x - data.nunchuk.js.center.x)) / data.nunchuk.js.max.x;
          if (movement > -30) {
            player_x[i]+= movement;
          }
        }
      }
    }

    if (player_x[i] < 0) player_x[i]= 0;
    if (player_x[i] > SCREEN_WIDTH - player_width) player_x[i]= SCREEN_WIDTH - player_width;
  }
}

/* draw all alive and exploding players */
void draw_players() {
  int i;
  for (i= 0; i < MAX_PLAYERS; i++) {
    if (player_status[i] == ALIVE_PLAYER) {
      GRRLIB_DrawImg(player_x[i], PLAYER_Y, player_image, 0, player_w_scale[level], 1, player_color[i]);
    } else if (player_status[i] >= EXPLODING_PLAYER && player_status[i] <= EXPLODED_PLAYER) {
      draw_explosion(player_x[i], PLAYER_Y, player_width, PLAYER_HEIGHT, (player_status[i] - EXPLODING_PLAYER) / EXPLOSION_IMAGE_REPEAT);
      if (player_status[i] == EXPLODED_PLAYER) {
        WPAD_Rumble(i, 0);
      }
      player_status[i]++;
    }
  }
}

/* return number of players killed by missile at (x,y) */
int hit_players(const int x, const int y) {
  int i;
  int killed= 0;

  for (i= 0; i < MAX_PLAYERS; i++) {
    if (player_status[i] == ALIVE_PLAYER &&
        x >= player_x[i] &&
        x <= player_x[i] + player_width &&
        y >= PLAYER_Y) {
      WPAD_Rumble(i, 1);
      player_status[i]= EXPLODING_PLAYER;
      killed++;
    }
  }

  if (killed) {
    play_explosion();
  }

  return killed;
}

/* return number of active players */
int active_players() {
  int i;
  int n_players= 0;
  for (i= 0; i < MAX_PLAYERS; i++) {
    if (player_status[i] > INACTIVE_PLAYER) {
      n_players++;
    }
  }
  return n_players;
}

