/*
Copyright (C) 2009 Marco Michelino <michelinux@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <string.h>
#include <grrlib.h>
#include <asndlib.h>
#include <ogc/conf.h>
#include "text.h"
#include "main.h"
#include "player.h"
#include "BMfont3.h"
#include "click_snd.h"

#define GRAY       0x808080FF
#define LIGHT_GRAY 0xC0C0C0FF

#define RUMBLE_DURATION 3

#define ENGLISH 0
#define ITALIAN 1
#define FRENCH  2
#define DUTCH   3
#define GERMAN  4

multilang_text title_text= {
  "FLYING SAUCERS",
  "DISCHI VOLANTI",
  "SOUCOUPES VOLANTES",
  "VLIEGENDE SCHOTELS",
  "FLIEGENDE UNTERTASSEN"
};

multilang_text levels_text[]= {
  {"LEVEL:", "LIVELLO:", "NIVEAU:", "NIVEAU:", "STUFE:"},
  {"EASY", "FACILE", "FACILE", "MAKKELIJK", "LEICHT"},
  {"NORMAL", "NORMALE", "NORMAL", "NORMAAL", "NORMAL"},
  {"HARD", "DIFFICILE", "DIFFICILE", "MOEILIJK", "SCHWER"},
};

multilang_text player1_movement_text[]= {
  {"PLAYER1:", "GIOCATORE1:", "JOUEUR1:", "SPELER1:", "SPIELER1:"},
  {"NUNCHUK", "NUNCHUK", "NUNCHUK", "NUNCHUK", "NUNCHUK"},
  {"BALANCE BOARD", "BALANCE BOARD", "BALANCE BOARD", "BALANCE BOARD", "BALANCE BOARD"}
};

multilang_text players_text[]= {
  {"PLAYERS:", "GIOCATORI:", "JOUEURS", "SPELERS:", "SPIELER:"},
  {"1", "1", "1", "1", "1"},
  {"2", "2", "2", "2", "2"},
  {"3", "3", "3", "3", "3"},
  {"4", "4", "4", "4", "4"}
};

multilang_text clouds_text[]= {
  {"CLOUDS:", "NUVOLE:", "NUAGES:", "WOLKEN:", "WOLKEN:"},
  {"NO", "NO", "NON", "NEE", "NEIN"},
  {"YES", "SI", "OUI", "JA", "JA"}
};

multilang_text sounds_text[]= {
  {"SOUNDS:", "SUONI:", "SONS:", "GELUID:", "SOUNDS:"},
  {"NO", "NO", "NON", "NEE", "NEIN"},
  {"YES", "SI", "OUI", "JA", "JA"}
};

multilang_text start_text[]= {
  {"START", "GIOCA", "COMMENCER", "BEGINNEN", "STARTEN"}
};

multilang_text connect_nunckuk_text= {
  "PLAYER%d CONNECT NUNCHUK TO WIIMOTE",
  "GIOCATORE%d COLLEGA IL NUNCHUK",
  "JOUEUR%d DOIT CONNECTER SA NUNCHUK",
  "SPELER%d SLUIT NUNCHUK AAN OP WIIMOTE",
  "SPIELER%d, BITTE NUNCHUK ANSCHLIESSEN"
};

multilang_text wiimote_not_detected_text= {
  "WIIMOTE %d NOT DETECTED",
  "WIIMOTE %d NON TROVATO",
  "WIIMOTE %d NON DETECTE",
  "WIIMOTE %d NIET GEDETECTEERD",
  "WII-FERNBEDIENUNG %d NICHT ERKANNT"
};

multilang_text back_text[]= {
  {"BACK", "INDIETRO", "RETOUR", "TERUG", "ZURUECK"}
};

multilang_text switch_on_bb_text= {
  "PLAYER1 SWITCH ON YOUR BALANCE BOARD",
  "GIOCATORE1 ACCENDI LA BALANCE BOARD",
  "JOUEUR1 CHANGE TA BALANCE BOARD",
  "SPELER1 ZET HET BALANCE BOARD AAN",
  "SPIELER1, BALANCE BOARD EINSCHALTEN"
};

multilang_text quit_play_again_text[]= {
  {"QUIT", "ESCI", "QUITTER", "AFSLUITEN", "BEENDEN"},
  {"PLAY AGAIN", "GIOCA ANCORA", "JOUER ENCORE", "SPEEL OPNIEUW", "ERNEUT SPIELEN"}
};

multilang_text quit_continue_text[]= {
  {"QUIT", "ESCI", "QUITTER", "VERLATEN", "BEENDEN"},
  {"CONTINUE", "CONTINUA", "CONTINUER", "VERDERGAAN", "WEITER"}
};

multilang_text pausing_text= {
  "PAUSING",
  "IN PAUSA",
  "EN PAUSE",
  "PAUZE",
  "PAUSE"
};

multilang_text game_over_text= {
  "GAME OVER",
  "PARTITA TERMINATA",
  "PARTIE TERMINEE",
  "SPEL VOORBIJ",
  "GAME OVER"
};

multilang_text saucers_destroyed_text= {
  "SAUCERS DESTROYED BY PLAYER%d: %d",
  "NEMICI ABBATTUTI DAL GIOCATORE%d: %d",
  "SOUCOUPE DETRUITE PAR JOUEUR%d: %d",
  "VLIEGENDE SCHOTELS VERNIETIGD DOOR SPELER%d: %d",
  "SPIELER%d: %d UNTERTASSEN VERNICHTET"
};

int language;

static int language_to_use[]= {
  ENGLISH, // if japanese speak english
  ENGLISH, // if english speak english
  GERMAN,  // if german speak german
  FRENCH,  // if french speak french
  ENGLISH, // if spanish speak english
  ITALIAN, // if italian speak italian
  DUTCH,   // if dutch speak dutch
  ENGLISH, // if simp. chinese speak english
  ENGLISH, // if trad. chinese speak english
  ENGLISH  // if korean speak english
};

static GRRLIB_texImg *tex_BMfont3= NULL;

static int last_rumble[MAX_PLAYERS];

/* inizialization for this module */
void init_text() {
  language= CONF_GetLanguage();
  if (language < 10) {
    language= language_to_use[language];
  } else {
    /* unknown language ??? */
    language= 0;
  }

  if (!tex_BMfont3) {
    tex_BMfont3= GRRLIB_LoadTexture(BMfont3);
    GRRLIB_InitTileSet(tex_BMfont3, FONT_SIZE, FONT_SIZE, 32);
  }
}

/* print a string at the center of givern row */
void draw_centered_string(int row, const char *text, u32 color) {
  int margin= (SCREEN_WIDTH - strlen(text) * FONT_SIZE) / 2;
  int y= 1.5 * FONT_SIZE * row + VERTICAL_BORDER;
  GRRLIB_Printf(margin, y, tex_BMfont3, color, 1, text);
}

/* print a string on the right of givern row */
void draw_right_aligned_string(int row, const char *text, u32 color) {
  int x= SCREEN_WIDTH - HORIZONTAL_BORDER - strlen(text) * FONT_SIZE;
  int y= 1.5 * FONT_SIZE * row + VERTICAL_BORDER;
  GRRLIB_Printf(x, y, tex_BMfont3, color, 1, text);
}

/* print a translated text at the center of givern row */
void draw_centered_text(int row, const multilang_text text, u32 color) {
  draw_centered_string(row, text[language], color);
}

/* inizialize a menu struct */
void init_horizontal_menu(int row, struct menu *m) {
  int len= 0;
  int i;

  for (i= 0; i < m->n; i++) {
    len+= strlen(m->text[i][language]);
  }
  len+= 2 * (m->n - 1);

  m->items[0].x= (SCREEN_WIDTH - len * FONT_SIZE) / 2;
  m->items[0].y= 1.5 * FONT_SIZE * row + VERTICAL_BORDER;
  m->items[0].x1 = m->items[0].x + strlen(m->text[0][language]) * FONT_SIZE;
  m->items[0].y1= FONT_SIZE * (1.5 * row + 1) + VERTICAL_BORDER;
  m->items[0].highlighted= 0;

  for (i= 1; i < m->n; i++) {
    m->items[i].x= m->items[i - 1].x1 + 2 * FONT_SIZE;
    m->items[i].y= m->items[0].y;
    m->items[i].x1= m->items[i].x + strlen(m->text[i][language]) * FONT_SIZE;
    m->items[i].y1= m->items[0].y1;
    m->items[i].highlighted= 0;
  }

  for (i= 0; i < MAX_PLAYERS; i++) {
    last_rumble[i]= RUMBLE_DURATION + 1;
  }
}

void draw_menu(int n, const struct menu m[]) {
  int i, j;
  for (j= 0; j < n; j++) {
    for (i= 0; i < m[j].n; i++) {
      if (m[j].selected + m[j].prompt == i || m[j].prompt > i) {
        GRRLIB_Printf(m[j].items[i].x, m[j].items[i].y, tex_BMfont3, WHITE, 1, m[j].text[i][language]);
      } else if (m[j].items[i].highlighted) {
        GRRLIB_Printf(m[j].items[i].x, m[j].items[i].y, tex_BMfont3, LIGHT_GRAY, 1, m[j].text[i][language]);
      } else {
        GRRLIB_Printf(m[j].items[i].x, m[j].items[i].y, tex_BMfont3, GRAY, 1, m[j].text[i][language]);
      }
    }
  }
}

/* highlight and select menu items */
void menu_interaction(int n, struct menu m[], WPADData *wd[], const u32 pressed[]) {
  int i, j, p, highlighted;

  for (j= 0; j < n; j++) {
    for (i= m[j].prompt; i < m[j].n; i++) {
      highlighted= 0;

      for (p= 0; p < MAX_PLAYERS; p++) {
        if (wd[p]) {
          if (m[j].items[i].x <= wd[p]->ir.x + 2 &&
              m[j].items[i].x1 >= wd[p]->ir.x - 2 &&
              m[j].items[i].y <= wd[p]->ir.y + 2 &&
              m[j].items[i].y1 >= wd[p]->ir.y - 2) {
            highlighted= 1;

            if (!m[j].items[i].highlighted && m[j].selected + m[j].prompt != i) {
              last_rumble[p]= 0;
              WPAD_Rumble(p, 1);
            }

            if (pressed[p] & (WPAD_BUTTON_A | WPAD_BUTTON_B)) {
              m[j].selected= i - m[j].prompt;
              WPAD_Rumble(p, 0);
              if (use_sounds) {
                ASND_SetVoice(ASND_GetFirstUnusedVoice(), VOICE_MONO_8BIT, 22050, 0, (char *) click_snd, click_snd_size, MID_VOLUME, MID_VOLUME, NULL);
              }
            }
          }
        }
      }

      m[j].items[i].highlighted= highlighted;
    }
  }

  for (p= 0; p < MAX_PLAYERS; p++) {
    if (last_rumble[p] <= RUMBLE_DURATION) {
      last_rumble[p]++;
      if (last_rumble[p] > RUMBLE_DURATION) {
       WPAD_Rumble(p, 0);
      }
    }
  }
}

